<?php
// 全てのプログラムファイルから参照される共通のファイル
// ログインチェックなどを行う
//------------------------------------------------------------------------------
// 初期設定
//------------------------------------------------------------------------------
// 言語の設定
mb_language("Japanese");
mb_internal_encoding("UTF-8");
// ユーザーの設定
$server_mail_from = "nadesiko_lang@yahoo.co.jp";             // サーバーメールアドレス★
// パスなど
$dbfile           = dirname(__FILE__)."/sikkari.db"; // データベースの保存先
$path_db_init     = dirname(__FILE__)."/db_init.sql";// DB定義ファイル
$script_name = $_SERVER["SCRIPT_NAME"];
// ライブラリの取り込み
include_once("lib/html-template.inc.php");
//------------------------------------------------------------------------------
// セッションを開始
session_start(); 
$db = new PDO("sqlite:".$dbfile); // データベースに接続
// データベースのチェック
$stmt = $db->query("SELECT * FROM sqlite_master");
$tables = $stmt->fetchAll();
if (count($tables) == 0) {
    // テーブルがなければテーブルを定義する
    $db->beginTransaction();
    $sql_text = file_get_contents($path_db_init); // テーブル定義ファイルの読込み
    $sql_a = explode(";", $sql_text); // セミコロンで１つずつの定義を分ける
    foreach($sql_a as $query) {       // 各定義を順に実行していく
        $r = $db->exec($query);       // テーブルの作成を実行
        if ($r === false) {
            echo "[データベースエラー]";
            print_r($db->errorInfo());
            $db->rollback();
            exit;
        }
    }
    $db->commit();
    // 続いて管理者の作成を行う
    header("location: ./login.php"); exit;
}

// データベースのオブジェクトを取得する関数★ ("s_"の接頭辞をつけておくと、入力補完が利いて便利)
function s_get_db() { global $db; return $db; }

// ログインチェック
if (!is_logined()) {
    // ログインしてなければログインページへジャンプ
    $base = basename($script_name);
    if ($base != "login.php") {
        $url = "./login.php?back=$script_name";
        header("location: $url"); exit;
    }
}

// ログインしているかどうか調べる関数
function is_logined() {
    return isset($_SESSION["login"]);
}

function is_admin() {
    if (!is_logined()) return false;
    $user = s_get_login_info();
    return ($user["user_type"] == "admin");
}

// 空白フィールドを削除して送信されたフォームデータの値を取得
function s_trim_get($name) {
    $v = isset($_REQUEST[$name]) ? $_REQUEST[$name] : ""; // パラメータ取得
    $v = trim($v); // トリム
    return $v;
}

// パスワードをハッシュ化する関数
function s_password_hash($password) {
    $salt = "bdrmegx8y8pzz3knr4u9hx6ks4rdp1de6k"; // 類推されにくい文字列を指定
    $pass_with_salt = $password.$salt;
    $hash = hash("sha512", $pass_with_salt);
    return $hash;
}

// サーバーからのメッセージ送信に利用
function s_sendmail($to, $subject, $message) {
    global $server_mail_from;
    $header = "From: $server_mail_from\r\n".
              "Replay-To: $server_mail_from\r\n".
              "X-Mailer: PHP/".  phpversion();
    $subject = "[シッカリスト]".$subject;
    return mb_send_mail($to, $subject, $message, $header);
}

// 管理人を取得する
function s_get_admin_user() {
    $db = s_get_db();
    $stmt = $db->query("SELECT * FROM users WHERE user_type='admin' LIMIT 1");
    return $stmt->fetch();
}

function s_get_user_info($user_id) {
    $db = s_get_db();
    $stmt = $db->query("SELECT * FROM users WHERE user_id=$user_id");
    return $stmt->fetch();
}

// エラーを画面に表示する
function s_error($message, $title = "エラー") {
    show_html_header();
    echo "<h3>$title</h3><div>$message</div>";
    show_html_footer();
    exit;
}

// メッセージを画面に表示する
function s_message($message, $title = "お知らせ") {
    show_html_header();
    echo "<h3>$title</h3><div>$message</div>";
    show_html_footer();
    exit;
}

// ログイン情報を得る
function s_get_login_info() {
    return isset($_SESSION["login"]) ? $_SESSION["login"] : array();
}
// ログイン中のユーザーのユーザーIDを得る
function s_get_user_id() {
    $info = s_get_login_info();
    return isset($info["user_id"]) ? intval($info["user_id"]) : 0;
}

// グループに関する情報を取得する
function s_get_group_info($group_id) {
    $group_id = intval($group_id);
    $q = "SELECT * FROM groups WHERE group_id=$group_id LIMIT 1";
    $db = s_get_db();
    $stmt = $db->query($q);
    $row = $stmt->fetch();
    return $row;
}

// 指定したグループに参加しているか調べる
function s_check_group_member($group_id, $user_id) {
    $db = s_get_db();
    $q = "SELECT * FROM group_members WHERE group_id=? AND user_id=? ";
    $r = $db->prepare($q);
    $r->execute(array($group_id, $user_id));
    $row = $r->fetch();
    return (isset($row["user_id"]));
}


// グループに属するTODOをHTMLで取得する
function s_get_group_todo($group_id, $status = 0, $limit = 100) {
    // TODOを取得する
    // (*1) SQLを作成する
    $q = "SELECT * FROM todo_list ".
        " WHERE status=$status AND group_id=$group_id ".
        " ORDER BY rank DESC, todo_id DESC LIMIT $limit";
    $item = "";
    // (*2) SQLを実行して結果を取り出す
    $db = s_get_db();
    $stmt = $db->query($q);
    // (*3) 取り出した結果を元にTODOの一覧をHTMLで作成
    foreach ($stmt as $row) {
        // (*4) カラムの値をHTMLに変換したりリンクを作ったりする
        $todo_id = intval($row["todo_id"]);
        $title_h = htmlspecialchars($row["title"]); // タイトル
        $date = date("m/d", $row["ctime"]);
        $user = s_get_user_info($row["user_id"]);
        $user_name_h = htmlspecialchars($user["nickname"]);
        $info = "<span class='item_info'>({$user_name_h} {$date})</span>";
        $rank = intval($row["rank"]);
        $star = show_todo_star($todo_id, $rank); // 重要度ランクのためのリンク
        if ($status == 0) {
            $link = "todo.php?m=done&todo_id=$todo_id&group_id=$group_id";
            $done_link = "<a href='$link'>完了</a>";
        } else {
            $link = "todo.php?m=readd&todo_id=$todo_id&group_id=$group_id";
            $done_link = "<a href='$link'>TODOに戻す</a>";
        }
        $div = "{$star} {$title_h} {$info} [$done_link]";
        $item .= "<div class='item'>$div</div>";
    }
    if ($item == "") {
        return "(ありません。)";
    }
    return $item;
}

function show_todo_star($todo_id, $rank) {
    $star = "";
    for ($i = 1; $i <= 5; $i++) {
        $link = "todo.php?m=setrank&todo_id=$todo_id&rank=$i";
        $star .= "<span class='star'><a href='$link'>";
        $star .= ($i <= $rank) ? "★" : "☆";
        $star .= "</a></span>";
    }
    return $star;
}

