<?php
// 共通設定を読み込む
include_once("common.inc.php");

$m = isset($_GET["m"]) ? $_GET["m"] : "";

// 管理ユーザーがいるかチェック
check_admin_user();
// モードごとに処理を分岐する
switch ($m) {
    case "try_login"    : try_login(); break;
    case "logout"       : m_logout(); break;
    case "signup"       : m_signup(); break;
    case "try_signup"   : m_try_signup(); break;
    case "emailcheck"   : m_emailcheck(); break;
    default:
        show_login_form($_GET);
        break;
}

function check_admin_user() {
    // 管理者のユーザーがいるか調べる
    $admin_user = get_admin_users();
    if (!$admin_user) { // 管理者のユーザーがいない時、管理ユーザーを作成
        $m = isset($_GET["m"]) ? $_GET["m"] : "";
        if ($m == "try_signup") { m_try_signup_admin_user(); }
        else { show_user_signup_form("管理ユーザー作成", "admin", array()); }
        exit;
    }
}

// 管理者ユーザーの一覧を得る
function get_admin_users() {
    $db = s_get_db();
    $stmt = $db->query("SELECT * FROM users WHERE user_type='admin'");
    return $stmt->fetchAll();
}

// ユーザー作成フォームを表示する
function show_user_signup_form($title, $user_type, $default, $errors = array()) {
    $self = $_SERVER["SCRIPT_NAME"];
    // 初期値のチェック
    $nickname  = isset($default["nickname"]) ? $default["nickname"]: "";
    $email = isset($default["email"]) ? $default["email"] : "";
    $nickname_h = htmlspecialchars($nickname, ENT_QUOTES);
    $email_h = htmlspecialchars($email, ENT_QUOTES);
    // 前回の入力でエラーがあった時の処理
    $error_str = implode($errors, "");
    show_html_header();
    echo <<< __FORM__
      <h3>$title</h3>
      <div id="user_form"><form action="$self" method="get">
        <input type="hidden" name="m" value="try_signup" />
        <input type="hidden" name="user_type" value="$user_type" />
        <label>ニックネーム</label>
        <input type="text" name="nickname" value="$nickname_h" />
        <label>メールアドレス</label>
        <input type="text" name="email" value="$email_h" />
        <label>パスワード</label>
        <input type="password" name="password" value="" />
        <label>パスワード(再入力)</label>
        <input type="password" name="password2" value="" />
        <label></label>
        <button type="submit">作成</button>
        <div style='color:red;padding:8px;'>$error_str</div>
      </form></div>
__FORM__;
    show_html_footer();
}

// 管理ユーザーの作成
function m_try_signup_admin_user() {
    // フォームの入力をチェック
    $errors = check_signup_form();
    // 何かしらのエラーがあった時★
    if (count($errors) > 0) {
        // フォームへ再入力してもらう
        show_user_signup_form("新規管理ユーザー作成", "admin", $_GET, $errors);
        exit;
    }
    // エラーがなければ結果を記憶
    $insert_query = <<< __SQL__
INSERT INTO users (nickname,email,password,user_type,agreement,ctime)
            VALUES(?,?,?,"admin",3,?);
__SQL__;
    $db = s_get_db();
    $stmt = $db->prepare($insert_query);
    $r = $stmt->execute(array(
        s_trim_get("nickname"), s_trim_get("email"),
        s_password_hash(s_trim_get("password")), time()
    ));
    if ($r === false) {
        echo "管理ユーザー作成に失敗:";
        print_r($db->errorInfo()); exit;
    }
    // 結果表示
    show_html_header();
    echo "<p><a href='login.php'>管理ユーザーを作成しました。</a></p>";
    show_html_footer();
}

// サインアップフォームをチェックしてエラーがあれば戻り値に配列を返す
function check_signup_form() {
    // フォームのパラメータを取得
    $nickname  = s_trim_get("nickname");
    $email     = s_trim_get("email");
    $password  = s_trim_get("password");
    $password2 = s_trim_get("password2");
    //
    // 入力エラーをチェック★
    $errors = array();
    if ($nickname == "") {
        $errors[] = "<p>ニックネームが空です。</p>";
    }
    if ($email == "") {
        $errors[] = "<p>メールが空です。</p>";
    } else {
        // 正規表現でメールアドレスの簡易チェック★
        if (!preg_match("/.+\@.+\..+/", $email)) {
            $errors[] = "<p>メールの形式に間違いがあります。</p>";
        }
    }
    if ($password == "") {
        $errors[] = "<p>パスワードが空です。</p>";
    } else if ($password != $password2) {
        $errors[] = "<p>再入力したパスワードが一致しません。</p>";
    }
    return $errors;
}

// 新規ユーザー登録
function m_signup() {
    show_user_signup_form("新規ユーザー登録", "normal", array());
}

// ユーザー登録チェック
function m_try_signup() {
    // 入力チェック
    $errors = check_signup_form();
    // エラーがあれば再入力
    if (count($errors) > 0) {
        show_user_signup_form("新規ユーザー登録", "normal", $_GET, $errors);
        exit;
    }
    $nickname = s_trim_get("nickname");
    $email = s_trim_get("email");
    $db = s_get_db();
    // 仮登録の状態で止まっている場合を考慮(メール認証が失敗する場合など)
    $stmt = $db->prepare(
            "SELECT * FROM users WHERE nickname=? AND email=? AND agreement!=3");
    $stmt->execute(array($nickname, $email));
    $row = $stmt->fetch();
    if (isset($row["user_id"])) {
        // 仮登録のユーザーを削除してやり直す
        $user_id = intval($row["user_id"]);
        $db->exec("DELETE FROM users WHERE user_id=$user_id");
    }

    // 既存ユーザーのニックネームが重複しないかチェック
    if ($nickname != "") {
        $stmt = $db->prepare("SELECT * FROM users WHERE nickname=? LIMIT 1");
        $stmt->execute(array($nickname));
        $row = $stmt->fetch();
        if (isset($row["nickname"])) {
            $errors[] = "<p>ニックネームが既に使用中です。</p>";
        }
    }
    // 既存ユーザーのメールアドレスが重複しないかチェック
    if ($email != "") {
        $stmt = $db->prepare("SELECT * FROM users WHERE email=? LIMIT 1");
        $stmt->execute(array($email));
        $row = $stmt->fetch();
        if (isset($row["email"])) {
            $errors[] = "<p>メールアドレスが既に使用中です。</p>";
        }
    }
    // エラーがあれば再入力
    if (count($errors) > 0) {
        show_user_signup_form("新規ユーザー登録", "normal", $_GET, $errors);
        exit;
    }
    // 仮登録する
    $insert_query = <<< __SQL__
INSERT INTO users (nickname,email,password,user_type,agreement,token,ctime)
            VALUES(?,?,?,"normal",0,?,?);
__SQL__;
    // メールアドレスを確認するためのランダムなトークンを生成
    $token = sprintf("%04d", rand(0, 9999)); 
    $stmt = $db->prepare($insert_query);
    $stmt->execute(array(
        $nickname, $email, s_password_hash(s_trim_get("password")),
        $token, time()
    ));
    $user_id = $db->lastInsertId(); // 今挿入したuser_idを取得する★
    // メールアドレスを確認する
    $host = $_SERVER["HTTP_HOST"];
    $self = $_SERVER["SCRIPT_NAME"];
    // メール本文
    $body = <<< __MAIL__
{$nickname} さん、シッカリストへのご登録ありがとうございます。
以下のURLにアクセスすれば、メールアドレスの認証が完了します。
↓
http://{$host}{$self}?m=emailcheck&user_id=$user_id&token=$token
__MAIL__;
    s_sendmail($email, "メール認証", $body);
    //
    show_html_header();
    echo "<h3>仮登録が完了しました。</h3>";
    echo "<p>メール認証用のメールを[$email]へ送信しました。</p>";
    echo "<p>メールにある認証用URLをクリックしてください。<p>";
    show_html_footer();
}

// メール認証
function m_emailcheck() {
    // パラメータを得る
    $user_id = intval(s_trim_get("user_id"));
    $token = s_trim_get("token"); // 4桁の数字(文字列)
    // データベースから登録情報を得る
    $db = s_get_db();
    $stmt = $db->prepare("SELECT * FROM users WHERE user_id=? AND token=?");
    $stmt->execute(array($user_id, $token));
    $row = $stmt->fetch();
    if (empty($row["user_id"])) { // 該当するユーザーがいない
        echo  "<p>認証用のトークンが間違っています。</p>"; exit;
    }
    else if ($row["agreement"] != 0) {
            echo "<p>既に認証が終わっています。</p>"; exit;
    }
    // メール認証を完了する
    $db->exec("UPDATE users SET agreement=1 WHERE user_id=$user_id");
    show_html_header();
    echo "<h3>メール認証が完了しました</h3>";
    echo "<p>管理人があなたを承認するのをしばらくお待ちください。</p>";
    show_html_footer();
    // ユーザーが新規登録されたことを管理人に通知する
    $nickname = $row["nickname"];
    $ctime    = $row["ctime"];
    $host = $_SERVER["HTTP_HOST"];
    $path = dirname($_SERVER["SCRIPT_NAME"]);
    $admin = s_get_admin_user();
    if (!$admin) return;
    $body = <<< __MAIL__
シッカリストに新規ユーザー[$nickname]さんが追加されました。
管理ページにアクセスしてユーザーを承認してください。
http://{$host}{$path}/admin.php
__MAIL__;
    s_sendmail($admin["email"], "承認してください", $body);
}

// ログインフォームを表示する
function show_login_form($default = array(), $errors = array()) {
    $self = $_SERVER["SCRIPT_NAME"];
    $error_str = implode("", $errors);
    $email_h = htmlspecialchars(s_trim_get("email"), ENT_QUOTES);
    $back = htmlspecialchars(s_trim_get("back"));
    show_html_header();
    echo <<< __FORM__
      <h3>ユーザーログイン</h3>
      <div id="user_form">
        <form action="$self" method="get">
        <input type="hidden" name="m" value="try_login" />
        <input type="hidden" name="back" value="$back" />
        <label>メールアドレス</label>
        <input type="text" name="email" value="$email_h" />
        <label>パスワード</label>
        <input type="password" name="password"  />
        <label></label>
        <button type="submit">ログイン</button>
        <div style='color:red;padding:8px;'>$error_str</div>
        <div><a href='login.php?m=signup'>→ユーザー新規登録</a></div>
      </form></div>
__FORM__;
    show_html_footer();
}

// ログインフォームで送信したパラメータを調べてログイン処理を行う
function try_login() {
    $email = s_trim_get("email");
    $password = s_trim_get("password");
    $password_hash = s_password_hash($password);
    $back = s_trim_get("back");
    // データベースと照合してログインするか判定する
    $db = s_get_db();
    $stmt = $db->prepare("SELECT * FROM users WHERE email=? AND password=?");
    $stmt->execute(array($email, $password_hash));
    $row = $stmt->fetch();
    if (empty($row["nickname"])) { // 一致結果がなければ再度入力
        show_login_form($_GET, array("メールかパスワードが正しくありません。"));
        exit;
    }
    if ($row["agreement"] != 3) {
        show_login_form($_GET, array("このユーザーは承認待ち中です。"));
        exit;
    }
    // ログインを記録
    $_SESSION["login"] = array(
        "user_id" => $row["user_id"],
        "nickname" => $row["nickname"],
        "email" => $row["email"],
        "user_type" => $row["user_type"],
        "login_time" => time()
    );
    if ($back == "") $back = "index.php";
    // ログインした旨を表示
    show_html_header("ログインしました");
    echo "<h3>ログイン成功</h3>";
    echo "<a href='$back'>ログインしました。</a>";
    show_html_footer();
}

function m_logout() {
    unset($_SESSION["login"]); // ログイン情報を削除
    header("location: login.php");// ログインページへジャンプ
}

