<?php
// 共通設定を読み込む
include_once("common.inc.php");
//パラメータ「m」に応じて処理を分岐させる
$m = isset($_GET["m"]) ? $_GET["m"] : "";
switch ($m) {
    case 'add'      : m_add(); break;    // TODOの新規追加
    case 'done'     : m_done(); break;   // TODOのステータスを完了にする
    case 'readd'    : m_readd(); break;  // 完了したTODOを再び追加する
    case 'setrank'  : m_setrank(); break;// 完了したTODOを再び追加する
    default         : show_todo();       // TODOの一覧を表示
}

// TODOを表示する処理
function show_todo() {
    $group_id = intval(s_trim_get("group_id"));
    if ($group_id <= 0) {
        s_error("グループの指定が必要です。");
    } else {
        show_group_todo($group_id); // グループ内のTODOを表示する
    }
}

// グループ内のTODOを表示する処理
function show_group_todo($group_id) {
    $self = $_SERVER["SCRIPT_NAME"];
    $group = s_get_group_info($group_id);
    if (!$group) {
        s_error("グループの指定が間違っています。"); exit;
    }
    $user_id = s_get_user_id();
    // TODOを取得
    $item = s_get_group_todo($group_id, 0, 100);
    // 完了済みアイテムを最大20件取得
    $done = s_get_group_todo($group_id, 1, 20);
    // 参加メンバーの一覧
    $members_h = "";
    $db = s_get_db();
    $stmt = $db->query("SELECT * FROM group_members WHERE group_id=$group_id");
    foreach ($stmt->fetchAll() as $p) {
        $user = s_get_user_info($p["user_id"]);
        $name_h = htmlspecialchars($user["nickname"]);
        $id = $user["user_id"];
        $members_h .= "<li><a href='index.php?user_id=$id'>$name_h</a></li>";
    }
    if ($members_h != "") { $members_h = "<ul>$members_h</ul>"; }
    // 表示
    $group_name_h = htmlspecialchars($group["name"]);
    show_html_header();
    echo "<h3>{$group_name_h}のTODO</h3>";
    if (s_check_group_member($group_id, $user_id)) {
        echo <<< __FORM__
<div id="todo_form">
<form action="$self" method="get">
    <input type="hidden" name="m" value="add" />
    <input type="hidden" name="group_id" value="$group_id" />
    TODO: <input type="text" name="title" value="" size="40" />
    <input type="submit" value="追加" />
</form>
</div>
__FORM__;
    } else {
        echo "<p>グループに参加するとTODOを追加できます。";
        echo "<a href='group.php?m=join&group_id=$group_id'>→参加</a></p>";
    }
    echo <<< __HTML__
<div class="todo_list">$item</div>

<div style="margin-left: 24px; margin-top:30px;">
    <h4>完了済みTODO</h4>
    <div class="done_list">$done</div>

    <h4>参加メンバー一覧</h4>
    <div>$members_h</div>
</div>
<hr/>
<p><a href="group.php">→グループ一覧に戻る</a></p>
__HTML__;
    show_html_footer();
}

// TODOを追加する処理
function m_add() {
    // パラメータのチェック
    $group_id = intval(s_trim_get("group_id"));
    $group = s_get_group_info($group_id);
    if (empty($group["name"])) {
        s_error("グループの指定がありません。"); exit;
    }
    $title = s_trim_get("title");
    if ($title == "") {
        s_error("内容が空です。"); exit;
    }
    // SQLの指定
    $insert = "INSERT INTO todo_list ".
        "(group_id,user_id,title,status,rank,ctime,mtime)".
        "VALUES(?,?,?,?,5,?,?)";
    // SQLを実行
    $db = s_get_db();
    $stmt = $db->prepare($insert);
    $user_id = s_get_user_id();
    $stmt->execute(array($group_id, $user_id, $title, 0, time(), time()));
    // 変更内容を示すためにページをリロードする
    $self = $_SERVER["SCRIPT_NAME"];
    header("location: $self?group_id=$group_id");
}

// TODOを完了する処理
function m_done() {
    $todo_id = intval(s_trim_get("todo_id"));
    $group_id = intval(s_trim_get("group_id"));
    $group = s_get_group_info($group_id);
    if (!$group) {
        s_error("グループがありません。"); exit;
    }
    $db = s_get_db();
    $p = $db->prepare("SELECT * FROM todo_list WHERE todo_id=? AND group_id=?");
    $p->execute(array($todo_id, $group_id));
    $r = $p->fetch();
    if (!$r) {
        s_error("TODOが見当たりません。"); exit;
    }
    // TODOのステータスを完了(1)にセット
    $t = time();
    $db->exec("UPDATE todo_list SET status=1, mtime=$t WHERE todo_id=$todo_id");
    // リロード
    $self = $_SERVER["SCRIPT_NAME"];
    header("location: $self?group_id=$group_id");
}

// 完了したTODOを再度TODOに戻す処理
function m_readd() {
    $todo_id = intval(s_trim_get("todo_id"));
    $group_id = intval(s_trim_get("group_id"));
    $group = s_get_group_info($group_id);
    if (!$group) {
        s_error("グループがありません。"); exit;
    }
    $db = s_get_db();
    $p = $db->prepare("SELECT * FROM todo_list WHERE todo_id=? AND group_id=?");
    $p->execute(array($todo_id, $group_id));
    $r = $p->fetch();
    if (!$r) {
        s_error("TODOが見当たりません。"); exit;
    }
    // TODOのステータスを完了(1)にセット
    $t = time();
    $db->exec("UPDATE todo_list SET status=0,mtime=$t WHERE todo_id=$todo_id");
    // リロード
    $self = $_SERVER["SCRIPT_NAME"];
    header("location: $self?group_id=$group_id");
}

// TODOのランクを変更する
function m_setrank() {
    $todo_id = intval(s_trim_get("todo_id"));
    $rank = intval(s_trim_get("rank"));
    if ($rank <= 0 || $rank > 5) {
        s_error("ランクの不正な操作"); exit;
    }
    $db = s_get_db();
    $p = $db->query("SELECT * FROM todo_list WHERE todo_id=$todo_id");
    $r = $p->fetch();
    if (!$r) {
        s_error("TODOがありません。"); exit;
    }
    $db->exec("UPDATE todo_list SET rank=$rank WHERE todo_id=$todo_id");
    // リロード
    $group_id = $r["group_id"];
    $self = $_SERVER["SCRIPT_NAME"];
    header("location: $self?group_id=$group_id");
}

